@@ -1,6 +1,6 @@
 /* GNOME-DB Components
  *
- * Copyright (C) 2000 - 2006 The GNOME Foundation.
+ * Copyright (C) 2000 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -22,7 +22,6 @@
  */
 
 #include <glib/gi18n-lib.h>
-#include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
 #include <libgda/gda-data-model-array.h>
 #include <gtk/gtkstock.h>
@@ -40,7 +39,6 @@ typedef struct {
 	GtkWidget *title;
 	GtkWidget *dsn_list;
 	GtkWidget *dialog;
-	guint      config_listener;
 } DsnConfigPrivate;
 
 static void
@@ -48,75 +46,14 @@ free_private_data (gpointer data)
 {
 	DsnConfigPrivate *priv = (DsnConfigPrivate *) data;
 
-	if (priv->config_listener != 0) {
-		gda_config_remove_listener (priv->config_listener);
-		priv->config_listener = 0;
-	}
-
 	g_free (priv);
 }
 
-static void
-show_data_sources (DsnConfigPrivate *priv)
-{
-	GList *dsn_list;
-	GList *l;
-	GdaDataModel *model, *current;
-	GtkTreeViewColumn *col;
-
-	/* create the data model */
-	model = gda_data_model_array_new_with_g_types (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
-	gda_data_model_set_column_title (model, 0, _("Name"));
-	gda_data_model_set_column_title (model, 1, _("Description"));
-	gda_data_model_set_column_title (model, 2, _("Global"));
-
-	dsn_list = gda_config_get_data_source_list ();
-	for (l = dsn_list; l != NULL; l = l->next) {
-		GList *value_list;
-		GdaDataSourceInfo *dsn_info = l->data;
-		GValue *tmpval;
-
-		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), dsn_info->name);
-		value_list = g_list_append (NULL, tmpval);
-
-		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), dsn_info->description);
-		value_list = g_list_append (value_list, tmpval);
-
-		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), dsn_info->is_global);
-		value_list = g_list_append (value_list, tmpval);
-
-		gda_data_model_append_values (model, (const GList *) value_list, NULL);
-
-		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
-		g_list_free (value_list);
-	}
-	g_object_set (G_OBJECT (model), "read_only", TRUE, NULL);
-	gda_config_free_data_source_list (dsn_list);
-
-	/* display the new data model */
-	current = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
-	if (current) {
-		g_object_set (G_OBJECT (current), "read_only", FALSE, NULL);
-		gda_data_model_import_from_model (current, model, TRUE, NULL, NULL);
-		g_object_set (G_OBJECT (current), "read_only", TRUE, NULL);
-	}
-	else
-		gnome_db_data_widget_set_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list), model);
-	g_object_unref (G_OBJECT (model));
-
-	col = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->dsn_list), 1);
-	g_object_set (G_OBJECT (col), "expand", TRUE, NULL);
-}
 
 /*
  * Callbacks
  */
 
-static void
-config_changed_cb (const gchar *path, gpointer user_data)
-{
-	show_data_sources (user_data);
-}
 
 static void
 list_double_clicked_cb (GnomeDbGrid *grid, gint row, gpointer user_data)
@@ -180,6 +117,7 @@ dsn_config_new (void)
 	GtkWidget *label;
 	GtkWidget *sw;
 	gchar *title;
+	GdaDataModel *model;
 
 	priv = g_new0 (DsnConfigPrivate, 1);
 	dsn = gnome_db_new_vbox_widget (FALSE, 6);
@@ -187,11 +125,6 @@ dsn_config_new (void)
 	g_object_set_data_full (G_OBJECT (dsn), DSN_CONFIG_DATA, priv,
 				(GDestroyNotify) free_private_data);
 
-	/* set up config listener */
-	priv->config_listener = gda_config_add_listener ("/apps/libgda/Datasources",
-							 (GdaConfigListenerFunc) config_changed_cb,
-							 priv);
-
 	/* create the main table */
 	table = gnome_db_new_table_widget (3, 1, FALSE);
 	gtk_box_pack_start (GTK_BOX (dsn), table, TRUE, TRUE, 0);
@@ -218,9 +151,13 @@ dsn_config_new (void)
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  0, 0);
 	
-	priv->dsn_list = gnome_db_raw_grid_new (NULL);
+	model = gda_config_list_dsn ();
+	priv->dsn_list = gnome_db_raw_grid_new (model);
+	g_object_unref (model);
 	g_object_set_data (G_OBJECT (dsn), "grid", priv->dsn_list);
  	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->dsn_list), 0, FALSE);
+ 	gnome_db_data_widget_column_hide (GNOME_DB_DATA_WIDGET (priv->dsn_list), 3);
+ 	gnome_db_data_widget_column_hide (GNOME_DB_DATA_WIDGET (priv->dsn_list), 4);
 	g_object_set (priv->dsn_list, "info_cell_visible", FALSE, NULL);
 
 	gtk_container_add (GTK_CONTAINER (sw), priv->dsn_list);
@@ -230,7 +167,6 @@ dsn_config_new (void)
 			  G_CALLBACK (list_double_clicked_cb), dsn);
 	g_signal_connect (priv->dsn_list, "populate_popup",
 			  G_CALLBACK (list_popup_cb), dsn);
-	show_data_sources (priv);
 
 	/* add tip */
 	box = gnome_db_new_hbox_widget (FALSE, 6);
@@ -268,6 +204,7 @@ dsn_config_edit_properties (GtkWidget *dsn)
 	GdaDataModel *model;
 	GList *selection;
 	gchar *str;
+	const GValue *cvalue;
 
 	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
 	
@@ -279,113 +216,69 @@ dsn_config_edit_properties (GtkWidget *dsn)
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
-	str = gda_value_stringify ((GValue *) gda_data_model_get_value_at (model, 0, GPOINTER_TO_INT (selection->data)));
+	cvalue = gda_data_model_get_value_at (model, 0, GPOINTER_TO_INT (selection->data), NULL);
+	if (!cvalue) 
+		return;
 
+	str = gda_value_stringify ((GValue *) cvalue);
 	dsn_properties_dialog (GTK_WINDOW (gtk_widget_get_toplevel (dsn)), str);
 
 	g_list_free (selection);
 	g_free (str);
 }
 
-static void
-foreach_remove_cb (GtkTreeModel *tree_model, GtkTreePath *path, GtkTreeIter *iter, DsnConfigPrivate *priv)
-{
-	GdaDataModel *model;
-	GtkWidget *dialog, *label;
-	gchar *str1, *str;
-	gchar *dsn;
-	gint row;
-
-	gtk_tree_model_get (tree_model, iter, DATA_STORE_COL_MODEL_ROW, &row, -1);
-	model = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
-	if (!GDA_IS_DATA_MODEL (model))
-		return;
-
-	dsn = gda_value_stringify ((GValue *) gda_data_model_get_value_at (model, 0, row));
-	if (dsn == NULL)
-		return;
-
-	dialog = gtk_dialog_new_with_buttons (_("Data source delete confirmation"),
-					      GTK_WINDOW (gtk_widget_get_toplevel (priv->dsn_list)),
-					      GTK_DIALOG_MODAL,
-					      GTK_STOCK_YES, GTK_RESPONSE_YES,
-					      GTK_STOCK_NO, GTK_RESPONSE_NO, NULL);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	str1 = g_strdup_printf (_("Are you sure you want to remove the data source '%s'?"), dsn);
-	str = g_strdup_printf ("<b>Data source removal confirmation:</b>\n\n%s", str1);
-	g_free (str1);
-
-	label = gtk_label_new ("");
-	gtk_label_set_markup (GTK_LABEL (label), str);
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, TRUE, TRUE, 10);
-					      
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-					 GTK_RESPONSE_NO);
-	gtk_widget_show (dialog);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES) 
-		gda_config_remove_data_source (dsn);
-
-	/* free memory */
-	gtk_widget_destroy (dialog);
-	g_free (dsn);
-	g_free (str);
-}
-
 void
 dsn_config_delete (GtkWidget *dsn)
 {
 	DsnConfigPrivate *priv;
 	GtkTreeSelection *sel;
+	GList *list, *sel_rows;
+	GList *sel_dsn = NULL;
+	GdaDataModel *model;
 
 	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
 
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->dsn_list));
+	sel_rows = gtk_tree_selection_get_selected_rows (sel, NULL);
+	model = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
+	g_assert (GDA_IS_DATA_MODEL (model));
 
-	gtk_tree_selection_selected_foreach (sel, 
-					     (GtkTreeSelectionForeachFunc) foreach_remove_cb, priv);
-	show_data_sources (priv);
-}
-
-void
-dsn_config_select_all (GtkWidget *dsn)
-{
-	DsnConfigPrivate *priv;
-	GtkTreeSelection *sel;
-
-	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
-
-	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->dsn_list));
-
-	gtk_tree_selection_select_all (sel);
-}
-
-void
-dsn_config_unselect_all (GtkWidget *dsn)
-{
-	DsnConfigPrivate *priv;
-	GtkTreeSelection *sel;
-
-	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
-
-	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->dsn_list));
-
-	gtk_tree_selection_unselect_all (sel);
-}
-
-gboolean
-dsn_config_has_selection (GtkWidget *dsn)
-{	
-	DsnConfigPrivate *priv;
-	GtkTreeSelection *sel;
-
-	priv = g_object_get_data (G_OBJECT (dsn), DSN_CONFIG_DATA);
-
-	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->dsn_list));
-
-	if (gtk_tree_selection_count_selected_rows (sel) >= 1)
-		return TRUE;
-	else
-		return FALSE;
+	/* make a list of DSN to remove */
+	for (list = sel_rows; list; list = list->next) {
+		gchar *dsn;
+		gint *row;
+		const GValue *cvalue;
+		
+		row = gtk_tree_path_get_indices ((GtkTreePath *) list->data);
+		cvalue = gda_data_model_get_value_at (model, 0, *row, NULL);
+		if (cvalue) {
+			dsn = gda_value_stringify ((GValue *) cvalue);
+			sel_dsn = g_list_prepend (sel_dsn, dsn);
+		}
+		gtk_tree_path_free ((GtkTreePath *) list->data);
+	}
+	g_list_free (sel_rows);
+
+	/* actually remove the DSN listed */
+	for (list = sel_dsn; list; list = list->next) {
+		gchar *str, *dsn;
+		GtkWidget *dialog;
+		dsn = (gchar *) list->data;
+
+		str = g_strdup_printf (_("Are you sure you want to remove the data source '%s'?"), dsn);
+		dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (priv->dsn_list)),
+							     GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+							     GTK_BUTTONS_YES_NO, 
+							     "<b>%s:</b>\n\n%s", _("Data source removal confirmation"),
+							     str);
+		g_free (str);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
+		gtk_widget_show (dialog);
+		
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES) 
+			gda_config_remove_dsn (dsn, NULL);
+		gtk_widget_destroy (dialog);
+		
+		g_free (dsn);
+	}
 }
