@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2007 The GNOME Foundation.
+ * Copyright (C) 1999 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -24,13 +24,17 @@
 #ifndef __LIBGNOME_DB_EXTRA_H_
 #define __LIBGNOME_DB_EXTRA_H_
 
-#include <libgnomedb-extra/gnome-db-dbms-update-viewer.h>
 #include <libgnomedb-extra/gnome-db-editor.h>
 #include <libgnomedb-extra/gnome-db-error-dialog.h>
 #include <libgnomedb-extra/gnome-db-error.h>
 #include <libgnomedb-extra/gnome-db-gray-bar.h>
 #include <libgnomedb-extra/gnome-db-report-editor.h>
-#include <libgnomedb-extra/gnome-db-selector.h>
 #include <libgnomedb-extra/gnome-db-sql-console.h>
+#include <libgnomedb-extra/gnome-db-selector.h>
+#include <libgnomedb-extra/gnome-db-selector-part.h>
+#include <libgnomedb-extra/gnome-db-selector-schema.h>
+#include <libgnomedb-extra/gnome-db-selector-meta.h>
+#include <libgnomedb-extra/gnome-db-selector-table.h>
+#include <libgnomedb-extra/gnome-db-selector-view.h>
 
 #endif
