@@ -133,7 +133,7 @@ gnome_db_format_entry_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo type_info = {
 			sizeof (GnomeDbFormatEntryClass),
 			NULL,		/* base_init */
@@ -1022,6 +1022,9 @@ adjust_numeric_display (GnomeDbFormatEntry *entry)
 	gchar *format;
 	NumAttr attr;
 
+	if (!entry->priv)
+		return FALSE;
+
 	compute_numeric_attributes (entry->priv->edited_type, &attr);
 	
 	if (!attr.is_numerical)
@@ -1128,7 +1131,7 @@ adjust_numeric_display (GnomeDbFormatEntry *entry)
 	g_free (new_text);
 
 	LEAVE_INTERNAL_CHANGES (entry);
-	g_signal_emit_by_name (entry, "changed");
+	//g_signal_emit_by_name (entry, "changed");
 
 	return FALSE; /* remove the timeout */
 }
